package javax.swing.text;

import java.awt.Color;
import javax.swing.UIManager;
import java.awt.event.ActionEvent;
import javax.swing.event.CaretEvent;
import java.beans.PropertyChangeEvent;
import java.io.Serializable;
import java.beans.PropertyChangeListener;
import javax.swing.event.CaretListener;
import javax.swing.JEditorPane;
import javax.swing.Action;

public class StyledEditorKit extends DefaultEditorKit
{
    private static final ViewFactory defaultFactory;
    Element currentRun;
    Element currentParagraph;
    MutableAttributeSet inputAttributes;
    private StyledEditorKit.AttributeTracker inputAttributeUpdater;
    private static final Action[] defaultActions;
    
    public StyledEditorKit() {
        this.createInputAttributeUpdated();
        this.createInputAttributes();
    }
    
    public MutableAttributeSet getInputAttributes() {
        return this.inputAttributes;
    }
    
    public Element getCharacterAttributeRun() {
        return this.currentRun;
    }
    
    @Override
    public Action[] getActions() {
        return TextAction.augmentList(super.getActions(), StyledEditorKit.defaultActions);
    }
    
    @Override
    public Document createDefaultDocument() {
        return new DefaultStyledDocument();
    }
    
    @Override
    public void install(JEditorPane editorPane) {
        editorPane.addCaretListener(this.inputAttributeUpdater);
        editorPane.addPropertyChangeListener(this.inputAttributeUpdater);
        Caret caret = editorPane.getCaret();
        if (caret != null) {
            this.inputAttributeUpdater.updateInputAttributes(caret.getDot(), caret.getMark(), editorPane);
        }
    }
    
    @Override
    public void deinstall(JEditorPane editorPane) {
        editorPane.removeCaretListener(this.inputAttributeUpdater);
        editorPane.removePropertyChangeListener(this.inputAttributeUpdater);
        this.currentRun = null;
        this.currentParagraph = null;
    }
    
    @Override
    public ViewFactory getViewFactory() {
        return StyledEditorKit.defaultFactory;
    }
    
    @Override
    public Object clone() {
        StyledEditorKit styledEditorKit2;
        StyledEditorKit styledEditorKit = styledEditorKit2 = (StyledEditorKit)super.clone();
        Element element = null;
        styledEditorKit.currentParagraph = element;
        styledEditorKit2.currentRun = element;
        styledEditorKit.createInputAttributeUpdated();
        styledEditorKit.createInputAttributes();
        return styledEditorKit;
    }
    
    private void createInputAttributes() {
        this.inputAttributes = new SimpleAttributeSet() {
            @Override
            public AttributeSet getResolveParent() {
                return (StyledEditorKit.this.currentParagraph != null) ? StyledEditorKit.this.currentParagraph.getAttributes() : null;
            }
            
            @Override
            public Object clone() {
                return new SimpleAttributeSet(this);
            }
        };
    }
    
    private void createInputAttributeUpdated() {
        this.inputAttributeUpdater = new StyledEditorKit.AttributeTracker();
    }
    
    protected void createInputAttributes(Element element, MutableAttributeSet set) {
        if (element.getAttributes().getAttributeCount() > 0 || element.getEndOffset() - element.getStartOffset() > 1 || element.getEndOffset() < element.getDocument().getLength()) {
            set.removeAttributes(set);
            set.addAttributes(element.getAttributes());
            set.removeAttribute(StyleConstants.ComponentAttribute);
            set.removeAttribute(StyleConstants.IconAttribute);
            set.removeAttribute("$ename");
            set.removeAttribute(StyleConstants.ComposedTextAttribute);
        }
    }
    
    static {
        defaultFactory = new StyledEditorKit.StyledViewFactory();
        defaultActions = new Action[] { new StyledEditorKit.FontFamilyAction("font-family-SansSerif", "SansSerif"), new StyledEditorKit.FontFamilyAction("font-family-Monospaced", "Monospaced"), new StyledEditorKit.FontFamilyAction("font-family-Serif", "Serif"), new StyledEditorKit.FontSizeAction("font-size-8", 8), new StyledEditorKit.FontSizeAction("font-size-10", 10), new StyledEditorKit.FontSizeAction("font-size-12", 12), new StyledEditorKit.FontSizeAction("font-size-14", 14), new StyledEditorKit.FontSizeAction("font-size-16", 16), new StyledEditorKit.FontSizeAction("font-size-18", 18), new StyledEditorKit.FontSizeAction("font-size-24", 24), new StyledEditorKit.FontSizeAction("font-size-36", 36), new StyledEditorKit.FontSizeAction("font-size-48", 48), new StyledEditorKit.AlignmentAction("left-justify", 0), new StyledEditorKit.AlignmentAction("center-justify", 1), new StyledEditorKit.AlignmentAction("right-justify", 2), new StyledEditorKit.BoldAction(), new StyledEditorKit.ItalicAction(), new StyledEditorKit.StyledInsertBreakAction(), new StyledEditorKit.UnderlineAction() };
    }
    
    class AttributeTracker implements CaretListener, PropertyChangeListener, Serializable
    {
        void updateInputAttributes(int a, int b, JTextComponent textComponent) {
            Document document = textComponent.getDocument();
            if (!(document instanceof StyledDocument)) {
                return;
            }
            int min = Math.min(a, b);
            StyledDocument styledDocument = (StyledDocument)document;
            StyledEditorKit.this.currentParagraph = styledDocument.getParagraphElement(min);
            Element currentRun;
            if (StyledEditorKit.this.currentParagraph.getStartOffset() == min || a != b) {
                currentRun = styledDocument.getCharacterElement(min);
            }
            else {
                currentRun = styledDocument.getCharacterElement(Math.max(min - 1, 0));
            }
            if (currentRun != StyledEditorKit.this.currentRun) {
                StyledEditorKit.this.currentRun = currentRun;
                StyledEditorKit.this.createInputAttributes(StyledEditorKit.this.currentRun, StyledEditorKit.this.getInputAttributes());
            }
            return;
        }
        
        @Override
        public void propertyChange(PropertyChangeEvent propertyChangeEvent) {
            Object newValue = propertyChangeEvent.getNewValue();
            Object source = propertyChangeEvent.getSource();
            if (source instanceof JTextComponent && newValue instanceof Document) {
                this.updateInputAttributes(0, 0, (JTextComponent)source);
            }
        }
        
        @Override
        public void caretUpdate(CaretEvent caretEvent) {
            this.updateInputAttributes(caretEvent.getDot(), caretEvent.getMark(), (JTextComponent)caretEvent.getSource());
        }
    }
    
    static class StyledViewFactory implements ViewFactory
    {
        @Override
        public View create(Element element) {
            String name = element.getName();
            if (name != null) {
                if (name.equals("content")) {
                    return new LabelView(element);
                }
                if (name.equals("paragraph")) {
                    return new ParagraphView(element);
                }
                if (name.equals("section")) {
                    return new BoxView(element, 1);
                }
                if (name.equals("component")) {
                    return new ComponentView(element);
                }
                if (name.equals("icon")) {
                    return new IconView(element);
                }
            }
            return new LabelView(element);
        }
    }
    
    public abstract static class StyledTextAction extends TextAction
    {
        public StyledTextAction(String s) {
            super(s);
        }
        
        protected final JEditorPane getEditor(ActionEvent actionEvent) {
            JTextComponent textComponent = this.getTextComponent(actionEvent);
            if ((textComponent instanceof JEditorPane)) {
                return (JEditorPane)textComponent;
            }
            return null;
        }
        
        protected final StyledDocument getStyledDocument(JEditorPane editorPane) {
            Document document = editorPane.getDocument();
            if ((document instanceof StyledDocument)) {
                return (StyledDocument)document;
            }
            throw new IllegalArgumentException("document must be StyledDocument");
        }
        
        protected final StyledEditorKit getStyledEditorKit(JEditorPane editorPane) {
            EditorKit editorKit = editorPane.getEditorKit();
            if ((editorKit instanceof StyledEditorKit)) {
                return (StyledEditorKit)editorKit;
            }
            throw new IllegalArgumentException("EditorKit must be StyledEditorKit");
        }
        
        protected final void setCharacterAttributes(JEditorPane editorPane, AttributeSet set, boolean b) {
            int selectionStart = editorPane.getSelectionStart();
            int selectionEnd = editorPane.getSelectionEnd();
            if (selectionStart != selectionEnd) {
                this.getStyledDocument(editorPane).setCharacterAttributes(selectionStart, selectionEnd - selectionStart, set, b);
            }
            MutableAttributeSet inputAttributes = this.getStyledEditorKit(editorPane).getInputAttributes();
            if (b) {
                inputAttributes.removeAttributes(inputAttributes);
            }
            inputAttributes.addAttributes(set);
        }
        
        protected final void setParagraphAttributes(JEditorPane editorPane, AttributeSet set, boolean b) {
            int selectionStart = editorPane.getSelectionStart();
            this.getStyledDocument(editorPane).setParagraphAttributes(selectionStart, editorPane.getSelectionEnd() - selectionStart, set, b);
        }
    }
    
    public static class FontFamilyAction extends StyledEditorKit.StyledTextAction
    {
        private String family;
        
        public FontFamilyAction(String s, String family) {
            super(s);
            this.family = family;
        }
        
        @Override
        public void actionPerformed(ActionEvent actionEvent) {
            JEditorPane editor = this.getEditor(actionEvent);
            if (editor != null) {
                String family = this.family;
                if (actionEvent != null && actionEvent.getSource() == editor) {
                    String actionCommand = actionEvent.getActionCommand();
                    if (actionCommand != null) {
                        family = actionCommand;
                    }
                }
                if (family != null) {
                    SimpleAttributeSet o = new SimpleAttributeSet();
                    StyleConstants.setFontFamily(o, family);
                    this.setCharacterAttributes(editor, o, false);
                }
                else {
                    UIManager.getLookAndFeel().provideErrorFeedback(editor);
                }
            }
        }
    }
    
    public static class FontSizeAction extends StyledEditorKit.StyledTextAction
    {
        private int size;
        
        public FontSizeAction(String s, int size) {
            super(s);
            this.size = size;
        }
        
        @Override
        public void actionPerformed(ActionEvent actionEvent) {
            JEditorPane editor = this.getEditor(actionEvent);
            if (editor != null) {
                int n = this.size;
                if (actionEvent != null && actionEvent.getSource() == editor) {
                    String actionCommand = actionEvent.getActionCommand();
                    try {
                        n = Integer.parseInt(actionCommand, 10);
                    }
                    catch (NumberFormatException ex) {}
                }
                if (n != 0) {
                    SimpleAttributeSet o = new SimpleAttributeSet();
                    StyleConstants.setFontSize(o, n);
                    this.setCharacterAttributes(editor, o, false);
                }
                else {
                    UIManager.getLookAndFeel().provideErrorFeedback(editor);
                }
            }
        }
    }
    
    public static class ForegroundAction extends StyledEditorKit.StyledTextAction
    {
        private Color fg;
        
        public ForegroundAction(String s, Color fg) {
            super(s);
            this.fg = fg;
        }
        
        @Override
        public void actionPerformed(ActionEvent actionEvent) {
            JEditorPane editor = this.getEditor(actionEvent);
            if (editor != null) {
                Color color = this.fg;
                if (actionEvent != null && actionEvent.getSource() == editor) {
                    String actionCommand = actionEvent.getActionCommand();
                    try {
                        color = Color.decode(actionCommand);
                    }
                    catch (NumberFormatException ex) {}
                }
                if (color != null) {
                    SimpleAttributeSet o = new SimpleAttributeSet();
                    StyleConstants.setForeground(o, color);
                    this.setCharacterAttributes(editor, o, false);
                }
                else {
                    UIManager.getLookAndFeel().provideErrorFeedback(editor);
                }
            }
        }
    }
    
    public static class AlignmentAction extends StyledEditorKit.StyledTextAction
    {
        private int a;
        
        public AlignmentAction(String s, int a) {
            super(s);
            this.a = a;
        }
        
        @Override
        public void actionPerformed(ActionEvent actionEvent) {
            JEditorPane editor = this.getEditor(actionEvent);
            if (editor != null) {
                int n = this.a;
                if (actionEvent != null && actionEvent.getSource() == editor) {
                    String actionCommand = actionEvent.getActionCommand();
                    try {
                        n = Integer.parseInt(actionCommand, 10);
                    }
                    catch (NumberFormatException ex) {}
                }
                SimpleAttributeSet o = new SimpleAttributeSet();
                StyleConstants.setAlignment(o, n);
                this.setParagraphAttributes(editor, o, false);
            }
        }
    }
    
    public static class BoldAction extends StyledEditorKit.StyledTextAction
    {
        public BoldAction() {
            super("font-bold");
        }
        
        @Override
        public void actionPerformed(ActionEvent actionEvent) {
            JEditorPane editor = this.getEditor(actionEvent);
            if (editor != null) {
                int n = !StyleConstants.isBold(this.getStyledEditorKit(editor).getInputAttributes());
                SimpleAttributeSet set = new SimpleAttributeSet();
                StyleConstants.setBold(set, (boolean)n);
                this.setCharacterAttributes(editor, set, false);
            }
        }
    }
    
    public static class ItalicAction extends StyledEditorKit.StyledTextAction
    {
        public ItalicAction() {
            super("font-italic");
        }
        
        @Override
        public void actionPerformed(ActionEvent actionEvent) {
            JEditorPane editor = this.getEditor(actionEvent);
            if (editor != null) {
                int n = !StyleConstants.isItalic(this.getStyledEditorKit(editor).getInputAttributes());
                SimpleAttributeSet set = new SimpleAttributeSet();
                StyleConstants.setItalic(set, (boolean)n);
                this.setCharacterAttributes(editor, set, false);
            }
        }
    }
    
    public static class UnderlineAction extends StyledEditorKit.StyledTextAction
    {
        public UnderlineAction() {
            super("font-underline");
        }
        
        @Override
        public void actionPerformed(ActionEvent actionEvent) {
            JEditorPane editor = this.getEditor(actionEvent);
            if (editor != null) {
                int n = !StyleConstants.isUnderline(this.getStyledEditorKit(editor).getInputAttributes());
                SimpleAttributeSet set = new SimpleAttributeSet();
                StyleConstants.setUnderline(set, (boolean)n);
                this.setCharacterAttributes(editor, set, false);
            }
        }
    }
    
    static class StyledInsertBreakAction extends StyledEditorKit.StyledTextAction
    {
        private SimpleAttributeSet tempSet;
        
        StyledInsertBreakAction() {
            super("insert-break");
        }
        
        @Override
        public void actionPerformed(ActionEvent actionEvent) {
            JEditorPane editor = this.getEditor(actionEvent);
            if (editor != null) {
                if (!editor.isEditable() || !editor.isEnabled()) {
                    UIManager.getLookAndFeel().provideErrorFeedback(editor);
                    return;
                }
                StyledEditorKit styledEditorKit = this.getStyledEditorKit(editor);
                if (this.tempSet != null) {
                    this.tempSet.removeAttributes(this.tempSet);
                }
                else {
                    this.tempSet = new SimpleAttributeSet();
                }
                this.tempSet.addAttributes(styledEditorKit.getInputAttributes());
                editor.replaceSelection("\n");
                MutableAttributeSet inputAttributes = styledEditorKit.getInputAttributes();
                inputAttributes.removeAttributes(inputAttributes);
                inputAttributes.addAttributes(this.tempSet);
                this.tempSet.removeAttributes(this.tempSet);
            }
            else {
                JTextComponent textComponent = this.getTextComponent(actionEvent);
                if (textComponent != null) {
                    if (!textComponent.isEditable() || !textComponent.isEnabled()) {
                        UIManager.getLookAndFeel().provideErrorFeedback(editor);
                        return;
                    }
                    textComponent.replaceSelection("\n");
                }
            }
        }
    }
}
